@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import
-
 from .application import Application
 from .config import Config, ConfigINI
 from .switches import (
@@ -19,3 +16,23 @@ from .switches import (
     positional,
     switch,
 )
+
+__all__ = (
+    "Application",
+    "Config",
+    "ConfigINI",
+    "CSV",
+    "CountOf",
+    "ExistingDirectory",
+    "ExistingFile",
+    "Flag",
+    "NonexistentPath",
+    "Predicate",
+    "Range",
+    "Set",
+    "SwitchAttr",
+    "SwitchError",
+    "autoswitch",
+    "positional",
+    "switch",
+)
