@@ -1,19 +1,10 @@
-# -*- coding: utf-8 -*-
-import IPython.display  # type: ignore
-from IPython.core.magic import magics_class  # type: ignore; type: ignore
-from IPython.core.magic import Magics, cell_magic, needs_local_scope
-
-try:
-    from io import StringIO
-except ImportError:
-    try:
-        from cStringIO import StringIO  # type: ignore
-    except ImportError:
-        from StringIO import StringIO  # type: ignore
-
 import sys
+from io import StringIO
 
-valid_choices = [x[8:] for x in dir(IPython.display) if "display_" == x[:8]]
+import IPython.display
+from IPython.core.magic import Magics, cell_magic, magics_class, needs_local_scope
+
+valid_choices = [x[8:] for x in dir(IPython.display) if x[:8] == "display_"]
 
 
 @magics_class
@@ -27,12 +18,12 @@ class OutputMagics(Magics):  # pragma: no cover
         )
         display_fn = getattr(IPython.display, "display_" + choice)
 
-        "Captures stdout and renders it in the notebook with some ."
+        # Captures stdout and renders it in the notebook
         with StringIO() as out:
             old_out = sys.stdout
             try:
                 sys.stdout = out
-                exec(cell, self.shell.user_ns, local_ns)
+                exec(cell, self.shell.user_ns, local_ns)  # pylint: disable=exec-used
                 out.seek(0)
                 display_fn(out.getvalue(), raw=True)
             finally:
