@@ -1,13 +1,11 @@
-# -*- coding: utf-8 -*-
+import contextlib
 import errno
 import logging
 import os
-import socket
 import stat
 
 from plumbum.commands.base import shquote
 from plumbum.commands.processes import ProcessLineTimedOut, iter_lines
-from plumbum.lib import _setdoc, six
 from plumbum.machines.base import PopenAddons
 from plumbum.machines.remote import BaseRemoteMachine
 from plumbum.machines.session import ShellSession
@@ -19,16 +17,14 @@ try:
     import paramiko
 except ImportError:
 
-    class paramiko(object):
-        def __nonzero__(self):
+    class paramiko:  # type: ignore[no-redef]
+        def __bool__(self):
             return False
 
-        __bool__ = __nonzero__
-
         def __getattr__(self, name):
             raise ImportError("No module named paramiko")
 
-    paramiko = paramiko()
+    paramiko = paramiko()  # type: ignore[operator]
 
 logger = logging.getLogger("plumbum.paramiko")
 
@@ -58,9 +54,8 @@ class ParamikoPopen(PopenAddons):
         self.stderr_file = stderr_file
 
     def poll(self):
-        if self.returncode is None:
-            if self.channel.exit_status_ready():
-                return self.wait()
+        if self.returncode is None and self.channel.exit_status_ready():
+            return self.wait()
         return self.returncode
 
     def wait(self):
@@ -75,11 +70,12 @@ class ParamikoPopen(PopenAddons):
         self.channel.shutdown_write()
         self.channel.close()
 
-    def kill(self):
+    @staticmethod
+    def kill():
         # possible way to obtain pid:
         # "(cmd ; echo $?) & echo ?!"
         # and then client.exec_command("kill -9 %s" % (pid,))
-        raise EnvironmentError("Cannot kill remote processes, we don't have their PIDs")
+        raise OSError("Cannot kill remote processes, we don't have their PIDs")
 
     terminate = kill
 
@@ -99,7 +95,7 @@ class ParamikoPopen(PopenAddons):
             if infile:
                 try:
                     line = infile.readline()
-                except (ValueError, IOError):
+                except (ValueError, OSError):
                     line = None
                 logger.debug("communicate: %r", line)
                 if not line:
@@ -111,7 +107,7 @@ class ParamikoPopen(PopenAddons):
                     self.stdin.flush()
 
             i = (i + 1) % len(sources)
-            name, coll, pipe, outfile = sources[i]
+            _name, coll, pipe, outfile = sources[i]
             line = pipe.readline()
             # logger.debug("%s> %r", name, line)
             if not line:
@@ -190,7 +186,7 @@ class ParamikoMachine(BaseRemoteMachine):
     :param bool load_system_ssh_config: read system SSH config for ProxyCommand configuration. default: False
     """
 
-    class RemoteCommand(BaseRemoteMachine.RemoteCommand):
+    class RemoteCommand(BaseRemoteMachine.RemoteCommand):  # type: ignore[valid-type, misc]
         def __or__(self, *_):
             raise NotImplementedError("Not supported with ParamikoMachine")
 
@@ -232,7 +228,7 @@ class ParamikoMachine(BaseRemoteMachine):
         self.host = host
         kwargs = {}
         if user:
-            self._fqhost = "{}@{}".format(user, host)
+            self._fqhost = f"{user}@{host}"
             kwargs["username"] = user
         else:
             self._fqhost = host
@@ -260,13 +256,11 @@ class ParamikoMachine(BaseRemoteMachine):
             kwargs["gss_host"] = gss_host
         if load_system_ssh_config:
             ssh_config = paramiko.SSHConfig()
-            with open(os.path.expanduser("~/.ssh/config")) as f:
+            with open(os.path.expanduser("~/.ssh/config"), encoding="utf-8") as f:
                 ssh_config.parse(f)
-            try:
+            with contextlib.suppress(KeyError):
                 hostConfig = ssh_config.lookup(host)
                 kwargs["sock"] = paramiko.ProxyCommand(hostConfig["proxycommand"])
-            except KeyError:
-                pass
         self._client.connect(host, **kwargs)
         self._keep_alive = keep_alive
         self._sftp = None
@@ -274,7 +268,7 @@ class ParamikoMachine(BaseRemoteMachine):
         BaseRemoteMachine.__init__(self, encoding, connect_timeout)
 
     def __str__(self):
-        return "paramiko://{}".format(self._fqhost)
+        return f"paramiko://{self._fqhost}"
 
     def close(self):
         BaseRemoteMachine.close(self)
@@ -290,9 +284,14 @@ class ParamikoMachine(BaseRemoteMachine):
             self._sftp = self._client.open_sftp()
         return self._sftp
 
-    @_setdoc(BaseRemoteMachine)
     def session(
-        self, isatty=False, term="vt100", width=80, height=24, new_session=False
+        self,
+        isatty=False,
+        term="vt100",
+        width=80,
+        height=24,
+        *,
+        new_session=False,  # noqa: ARG002
     ):
         # new_session is ignored for ParamikoMachine
         trans = self._client.get_transport()
@@ -308,14 +307,13 @@ class ParamikoMachine(BaseRemoteMachine):
         proc = ParamikoPopen(["<shell>"], stdin, stdout, stderr, self.custom_encoding)
         return ShellSession(proc, self.custom_encoding, isatty)
 
-    @_setdoc(BaseRemoteMachine)
     def popen(
         self,
         args,
         stdin=None,
         stdout=None,
         stderr=None,
-        new_session=False,
+        new_session=False,  # noqa: ARG002
         env=None,
         cwd=None,
     ):
@@ -327,7 +325,7 @@ class ParamikoMachine(BaseRemoteMachine):
         argv.extend(["cd", str(cwd or self.cwd), "&&"])
         if envdelta:
             argv.append("env")
-            argv.extend("{}={}".format(k, shquote(v)) for k, v in envdelta.items())
+            argv.extend(f"{k}={shquote(v)}" for k, v in envdelta.items())
         argv.extend(args.formulate())
         cmdline = " ".join(argv)
         logger.debug(cmdline)
@@ -343,14 +341,13 @@ class ParamikoMachine(BaseRemoteMachine):
             stderr_file=stderr,
         )
 
-    @_setdoc(BaseRemoteMachine)
     def download(self, src, dst):
         if isinstance(src, LocalPath):
-            raise TypeError("src of download cannot be {!r}".format(src))
+            raise TypeError(f"src of download cannot be {src!r}")
         if isinstance(src, RemotePath) and src.remote != self:
-            raise TypeError("src {!r} points to a different remote machine".format(src))
+            raise TypeError(f"src {src!r} points to a different remote machine")
         if isinstance(dst, RemotePath):
-            raise TypeError("dst of download cannot be {!r}".format(dst))
+            raise TypeError(f"dst of download cannot be {dst!r}")
         return self._download(
             src if isinstance(src, RemotePath) else self.path(src),
             dst if isinstance(dst, LocalPath) else LocalPath(dst),
@@ -367,14 +364,13 @@ class ParamikoMachine(BaseRemoteMachine):
         else:
             self.sftp.get(str(src), str(dst))
 
-    @_setdoc(BaseRemoteMachine)
     def upload(self, src, dst):
         if isinstance(src, RemotePath):
-            raise TypeError("src of upload cannot be {!r}".format(src))
+            raise TypeError(f"src of upload cannot be {src!r}")
         if isinstance(dst, LocalPath):
-            raise TypeError("dst of upload cannot be {!r}".format(dst))
+            raise TypeError(f"dst of upload cannot be {dst!r}")
         if isinstance(dst, RemotePath) and dst.remote != self:
-            raise TypeError("dst {!r} points to a different remote machine".format(dst))
+            raise TypeError(f"dst {dst!r} points to a different remote machine")
         return self._upload(
             src if isinstance(src, LocalPath) else LocalPath(src),
             dst if isinstance(dst, RemotePath) else self.path(dst),
@@ -423,7 +419,7 @@ class ParamikoMachine(BaseRemoteMachine):
         return data
 
     def _path_write(self, fn, data):
-        if self.custom_encoding and isinstance(data, six.unicode_type):
+        if self.custom_encoding and isinstance(data, str):
             data = data.encode(self.custom_encoding)
         f = self.sftp.open(str(fn), "wb")
         f.write(data)
@@ -432,10 +428,10 @@ class ParamikoMachine(BaseRemoteMachine):
     def _path_stat(self, fn):
         try:
             st = self.sftp.stat(str(fn))
-        except IOError as e:
+        except OSError as e:
             if e.errno == errno.ENOENT:
                 return None
-            raise OSError(e.errno)
+            raise
         res = StatRes(
             (
                 st.st_mode,
@@ -457,12 +453,15 @@ class ParamikoMachine(BaseRemoteMachine):
             res.text_mode = "regular file"
         return res
 
+    def daemonic_popen(self, command, cwd="/", stdout=None, stderr=None, append=True):
+        raise NotImplementedError("This is not implemented on ParamikoMachine!")
+
 
 ###################################################################################################
 # Make paramiko.Channel adhere to the socket protocol, namely, send and recv should fail
 # when the socket has been closed
 ###################################################################################################
-class SocketCompatibleChannel(object):
+class SocketCompatibleChannel:
     def __init__(self, chan):
         self._chan = chan
 
@@ -471,53 +470,40 @@ class SocketCompatibleChannel(object):
 
     def send(self, s):
         if self._chan.closed:
-            raise socket.error(errno.EBADF, "Bad file descriptor")
+            raise OSError(errno.EBADF, "Bad file descriptor")
         return self._chan.send(s)
 
     def recv(self, count):
         if self._chan.closed:
-            raise socket.error(errno.EBADF, "Bad file descriptor")
+            raise OSError(errno.EBADF, "Bad file descriptor")
         return self._chan.recv(count)
 
 
 ###################################################################################################
 # Custom iter_lines for paramiko.Channel
 ###################################################################################################
-def _iter_lines(proc, decode, linesize, line_timeout=None):
+def _iter_lines(
+    proc,
+    decode,  # noqa: ARG001
+    linesize,
+    line_timeout=None,
+):
+    from selectors import EVENT_READ, DefaultSelector
 
-    try:
-        from selectors import EVENT_READ, DefaultSelector
-    except ImportError:
-        # Pre Python 3.4 implementation
-        from select import select
-
-        def selector():
-            while True:
-                rlist, _, _ = select([proc.stdout.channel], [], [], line_timeout)
-                if not rlist and line_timeout:
-                    raise ProcessLineTimedOut(
-                        "popen line timeout expired",
-                        getattr(proc, "argv", None),
-                        getattr(proc, "machine", None),
-                    )
-                for _ in rlist:
-                    yield
-
-    else:
-        # Python 3.4 implementation
-        def selector():
-            sel = DefaultSelector()
-            sel.register(proc.stdout.channel, EVENT_READ)
-            while True:
-                ready = sel.select(line_timeout)
-                if not ready and line_timeout:
-                    raise ProcessLineTimedOut(
-                        "popen line timeout expired",
-                        getattr(proc, "argv", None),
-                        getattr(proc, "machine", None),
-                    )
-                for key, mask in ready:
-                    yield
+    # Python 3.4+ implementation
+    def selector():
+        sel = DefaultSelector()
+        sel.register(proc.stdout.channel, EVENT_READ)
+        while True:
+            ready = sel.select(line_timeout)
+            if not ready and line_timeout:
+                raise ProcessLineTimedOut(
+                    "popen line timeout expired",
+                    getattr(proc, "argv", None),
+                    getattr(proc, "machine", None),
+                )
+            for _key, _mask in ready:
+                yield
 
     for _ in selector():
         if proc.stdout.channel.recv_ready():
